#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2013 Gary Wright wrightsolutions.co.uk/contact
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# fab -a -f debian.py -u root -H aa.bb.cc.dd sshd_summary
# fab -a -f debian.py -u root -H aa.bb.cc.dd netselect_apt
# fab -a -f debian.py -u root -H aa.bb.cc.dd nameserver_check
# fab -a -f debian.py -u root -H aa.bb.cc.dd nameserver_check:domain=wrightsolutions.co.uk

import fabutil as futil
from fabutil import conf_get_set as getset


def print_preamble(backports_epel_report=True):
    """ Even if we know we are Debian or RPM based, we might
    still distinguish servers based on availability of extra repositories.
    backports_epel_true=True (default) gives you some feedback about extra
    repositories.
    """
    pretty_tuple = futil.cmdbin.pretty_name_tuple()
    print pretty_tuple
    if futil.cmdbin.cpu64():
        print "CPU has 64 bit support"
    if pretty_tuple[1] == 'debian':
        print "Backports: {0}".format(futil.package_deb.has_backports())
    elif pretty_tuple[2] in futil.cmdbin.NUM_AS_WORDS_TO_TWENTY:
        print "EPEL: {0}".format(futil.package_rpm.has_epel())
        print futil.package_rpm.repolist_dict()
    return


def sshd_summary():
    for line in futil.ssh_lokkit.sshd_config_keylines_enabled():
        print line
    #print futil.ssh_lokkit.sshd_config_keylines_enabled_linesep()
    print futil.ssh_lokkit.sshd_motd_yes_preview()
    return


def iptables_save():
    return futil.iptables_firewall.iptables_save()


def iptables_save_verbose():
    return futil.iptables_firewall.iptables_save(verbose_counts=True)


def sysstat_regular():
    conf_filepath = '/etc/default/sysstat'
    with getset.Keyedfile(conf_filepath) as conf:
        if 'ENABLED' in conf:
            #print conf.toggle_safe('ENABLED','true',True,1)
            if conf.toggle_safe('ENABLED','"true"',True):
                sedlines = conf.list_of_sedlines
                getset.sed_remote_apply('a'*32,conf_filepath,sedlines,'medium')
    return


def sysstat_regular_preview():
    """ Verbose / debug version of sysstat_regular() which supplements
    the test suite """
    with getset.Keyedfile('/etc/default/sysstat') as conf:
        #print conf.tuples_all
        #return conf.keylines_typed(2,['ENABLED'])
        if 'ENABLED' in conf:
            #val = conf['ENABLED']
            print conf.keylines(['ENABLED'])
            #if val in conf.FALSELIST
            print conf.tuples_all_from_key('ENABLED')
            print "Expect next 3 booleans to read False,True,False"
            print conf.toggle_safe('ENABLED','true',False)
            print conf.toggle_safe('ENABLED','true',True,1) # (says safe=True and autoquote_level=1)
            print conf.toggle_safe('ENABLED','true',False)
            #for sed in conf.list_of_sedlines: print sed
            for linenum,sed in conf.dict_of_seds.iteritems():
                print "{0}    {1}".format(linenum,sed)
            lines_current = conf.lines_current_simple()
            print len(lines_current)
            print "Above was lines_current ; next is put_lines"
            put_lines = conf.lines_for_put()
            print len(put_lines)
            scriptlines,md5after,lines = conf.sed_checksum(conf.list_of_sedlines,
                                                           debug_filename='sysstat_regular_preview')
            if md5after is not None:
                print md5after
            conf.dequeue_all()
            print len(conf.dict_of_seds)
            #print conf.sedline('ENABLED','yes')
    return


def netselect_apt():
    print_preamble()
    return futil.package_deb.netselect_apt()


def apache2ports():
    """ conf_get_set has no concept of nesting or sections so is probably not
    well suited to trying to toggle values in apache2 configurations.
    However if you want to just treat the whole file as a flat structure and
    ask for what conf_get_set thinks is enabled, then what you will see
    returned is the LAST value of Listen and the LAST value of NameVirtualHost
    """
    with getset.Keyedfile('/etc/apache2/ports.conf') as conf:
        if 'Listen' in conf.items():
            print conf['Listen']
        print "Now casting net wider we obtain the following"
        #print conf.keylist_frozen
        for conf_k in conf:
            print conf_k
            #print conf[conf_k]
        print conf.keylines_typed(1,['Listen'])
        for line_listen in conf.lines_enabled_contains('Listen'):
            print line_listen
        #print conf.keylines_typed(2,['NameVirtualHost'])
        for tup in conf.tuples_all:
            if tup[0] is not None and 'Listen' in tup[0]:
                print tup


def nameserver_check(domain=None,verbose=False):
    futil.dns.nameserver_count_excluding(futil.dns.GOOGLELIST,True)
    #futil.dns.nameserver_count_excluding('google')
    if verbose:
        futil.dns.resolvconf_dirs_more()
    futil.dns.nameserver_count_startswith('192.',True,True)
    return

