#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2013 Gary Wright wrightsolutions.co.uk/contact
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# Tested against fabric 1.4.3
# http://ftp.uk.debian.org/debian/pool/main/f/fabric/fabric_1.4.3-1.debian.tar.gz

""" Locale specifically and anything related to language / encoding / localisation """

from __future__ import with_statement
from fabric.api import abort, cd, env, get, hide, put, run, settings, sudo
#from fabric.contrib.console import confirm
from fabric.contrib.files import exists,append
from os import path
import re
from . import cmdbin
from . import conf_get_set as getset

CMD_LOCALE="/usr/bin/locale "
CMD_LOCALE_LC_CTYPE="/usr/bin/locale -c LC_CTYPE "
CMD_LOCALE_UPDATE="/usr/sbin/update-locale "

LOCALE_DEFAULT_CONF='/etc/default/locale'


def lang_lc_short(verbose=False):
    """ Returns an OrderedDict representation of locale
    limited to just two entries maximum - LANG and LC_CTYPE
    """
    # chr(58) is colon (:)            ; chr(61) is equals (=) ; chr(32) is space
    loclines = []
    if verbose == True:
        locale_res = cmdbin.run(CMD_LOCALE)
        loclines = cmdbin.run_res_lines_stripped(locale_res)
    else:
        with hide('output'):
            locale_res = cmdbin.run(CMD_LOCALE)
            loclines = cmdbin.run_res_lines_stripped(locale_res)
    locale_dict = {}
    for line in loclines:
        if line.startswith('LC_CTYPE='):
            locale_dict['LC_CTYPE']=line.split(chr(61))[1]
        elif line.startswith('LANG='):
            locale_dict['LANG']=line.split(chr(61))[1]
        else:
            pass
    return locale_dict


def lang_lc_only(verbose=False):
    """ Returns an OrderedDict representation of locale
    with key entries populated for all locale output
    beginning LC_
    For clarity: There would normally be more than two key entries
    in the returned dict. If you are only interested in LANG and LC_CTYPE
    specifically, then see lang_lc_short()
    LANG key would not appear in the results from this function.
    """
    # chr(58) is colon (:)            ; chr(61) is equals (=) ; chr(32) is space
    loclines = []
    if verbose == True:
        locale_res = cmdbin.run(CMD_LOCALE)
        loclines = cmdbin.run_res_lines_stripped(locale_res)
    else:
        with hide('output'):
            locale_res = cmdbin.run(CMD_LOCALE)
            loclines = cmdbin.run_res_lines_stripped(locale_res)
    locale_dict = {}
    for line in loclines:
        if line.startswith('LC_') and chr(61) in line:
            line_array = line.split(chr(61))
            locale_dict[line_array[0]]=line_array[1]
    return locale_dict

    
def lang_lc_ctype_report(missing_shows_fail=False):
    """ Report the content of LANG and LC_CTYPE.
    If one or other does not exist and missing_shows_fail is 
    True then return False to caller.
    """
    # chr(58) is colon (:)            ; chr(61) is equals (=) ; chr(32) is space
    report_return = True
    locale_dict_short = lang_lc_short()
    key_hits = 0
    if 'LANG' in locale_dict_short:
        key_hits += 1
        print "Locale LANG is {0}".format(locale_dict_short['LANG'])
    elif missing_shows_fail == True:
        report_return = False
    else:
        pass
    if 'LC_CTYPE' in locale_dict_short:
        key_hits += 1
        print "Locale LC_CTYPE is {0}".format(locale_dict_short['LC_CTYPE'])
    elif missing_shows_fail == True:
        report_return = False
    else:
        pass
    return report_return


