#!/usr/bin/env python
# -*- coding: utf-8 -*-
#
#   Copyright 2013 Gary Wright wrightsolutions.co.uk/contact
#
#   Licensed under the Apache License, Version 2.0 (the "License");
#   you may not use this file except in compliance with the License.
#   You may obtain a copy of the License at
#
#       http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS,
#   WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
#   See the License for the specific language governing permissions and
#   limitations under the License.

# Tested against fabric 1.4.3
# http://ftp.uk.debian.org/debian/pool/main/f/fabric/fabric_1.4.3-1.debian.tar.gz

from __future__ import with_statement
from fabric.api import abort, cd, env, get, put, run, settings, sudo
from fabric.contrib.console import confirm
from fabric.contrib.files import exists,append
from os import path
import re
from . import cmdbin
from . import conf_get_set as getset


def max3(auth_tries=6,sessions=10,startups='10:30:100'):
    auth_tries_changed = False
    sessions_changed = False
    startups_changed = False
    return (auth_tries_changed,sessions_changed,startups_changed)


def max3return():
    with getset.Keyedfile('/etc/ssh/sshd_config') as conf:
        #print conf.keylines_typed(2,['MaxAuthTries','MaxSessions','MaxStartups'])
        print conf.linecount
        print conf.md5sum
        print conf.lines
        return conf.keylines(['MaxAuthTries','MaxSessions','MaxStartups'])

def max3return2():
    with getset.Keyedfile('/etc/ssh/sshd_config') as conf:
        #print conf.keylines(['MaxAuthTries','MaxSessions','MaxStartups'])
        return conf.keylines_typed(2,['MaxAuthTries','MaxSessions','MaxStartups'])


def sshd_config_keylines_enabled(linesep_remove=False):
    """ Returns important content from /etc/ssh/sshd_config - namely those lines
    of the form key: val and excludes #key: val 
    """
    enabled = []
    with getset.Keyedfile('/etc/ssh/sshd_config') as conf:
        enabled = conf.keylines_enabled_resplit()
    return enabled

    
def sshd_config_keylines_enabled_linesep(linesep_remove=False):
    """ Returns important content from /etc/ssh/sshd_config - namely those lines
    of the form key: val and excludes #key: val 
    """
    with getset.Keyedfile('/etc/ssh/sshd_config') as conf:
        enabled = conf.keylines_enabled()
        if linesep_remove:
            unsplit = ''.join(enabled)
            #print "conf_linesep:{0} {0} {0}".format(conf.conf_linesep)
            returned = unsplit.split(conf.conf_linesep)
        else:
            returned = enabled
    return returned


def sshd_motd_yes_preview():
    preview_line = ''
    with getset.Keyedfile('/etc/ssh/sshd_config') as conf:
        preview_line = conf.sedline('PrintMotd','yes')
    return preview_line


""" Openssh 5.3 and newer defaults for Max type settings:
MaxAuthTries
   Specifies the maximum number of authentication attempts permitted per connection.  Once the
   number of failures reaches half this value, additional failures are logged.  The default is 6.

MaxSessions
   Specifies the maximum number of open sessions permitted per network connection.  The default is 10.

MaxStartups
   Specifies the maximum number of concurrent unauthenticated connections to the SSH daemon.
   Additional connections will be dropped until authentication succeeds or the LoginGraceTime
   expires for a connection.  The default is 10:30:100.

   Alternatively, random early drop can be enabled by specifying the three colon separated values
   “start:rate:full” (e.g. "10:30:60").  sshd(8) will refuse connection attempts with a
   probability of “rate/100” (30%) if there are currently “start” (10) unauthenticated connections.
   The probability increases linearly and all connection attempts are refused if
   the number of unauthenticated connections reaches “full” (60).
"""
